#version 330

in vec3 vNormal;
in vec3 vPosition;
in vec2 vTexCoord;

out vec4 fragColor;

uniform vec4 lightPosition;
uniform vec3 lightAmbientColor;
uniform vec3 lightDiffuseColor;
uniform vec3 lightSpecularColor;

uniform vec3 eyePosition;

uniform vec3 materialAmbientColor;
uniform vec3 materialDiffuseColor;
uniform vec3 materialSpecularColor;
uniform float materialShininess;

vec3 phong(vec3 matAmbientColor, vec3 matDiffuseColor, vec3 matSpecularColor, float matShininess);

// Use these if your GPU driver crashes with smoothstep
vec2 smoothstep_impl(float edge0, float edge1, vec2 x) {
    vec2 t = clamp((x - edge0) / (edge1 - edge0), 0.0, 1.0);
    return t * t * (3.0 - 2.0 * t);
}

float smoothstep_impl(float edge0, float edge1, float x) {
    float t = clamp((x - edge0) / (edge1 - edge0), 0.0, 1.0);
    return t * t * (3.0 - 2.0 * t);
}

const vec3 orange = vec3(0.81, 0.32, 0.0);
const vec3 grey = vec3(0.6602, 0.6562, 0.6445);
const vec3 red = vec3(1.0, 0.0, 0.0);
const vec3 white = vec3(1.0);

void main() {
    vec3 color;
    // Task 1:  make the surface color change in horizontal gradient from grey to orange
    //              use mix(x, y, a) http://docs.gl/sl4/mix
    // Task 2:  make the pattern repeat 4 times on the surface
    //              use fract(x) http://docs.gl/sl4/fract
    // Task 3:  make the color change suddenly at 0.2 coordinate: [0, 0.2) -> grey, [0.2, 1.0] -> orange
    //              use step(edge, x) http://docs.gl/sl4/step
    // Task 4:  make the transition smoother: [0, 0.1] -> grey, (0.1, 0.2) -> smooth gradient, [0.2, 1.0] -> orange
    //              use smoothstep(edge0, edge1, x) http://docs.gl/sl4/smoothstep (if that crashes your program, use smoothstep_impl)
    // Task 5:  make the transition on the other side too: [0, 0.1] -> grey, (0.1, 0.2) -> smooth gradient, [0.2, 0.8] -> orange, (0.8, 0.9) -> smooth gradient, [0.9, 1.0] -> grey
    // Task 6:  expand the pattern to both dimensions to create bricks. remember that most functions can work with vectors too.
    // Task 7:  shift even brick rows by 0.5 to the right to create an alternating brick pattern, similiar to a real wall
    //              you *could* use mod(x, y) http://docs.gl/sl4/mod (but you don't have to)
    // Task 8:  remove/comment out all the previously written code for brick pattern generation
    //vec2 repeat = vec2(4.0, 8.0);
    //float offset = floor(repeat.t * vTexCoord.t);
    //vec2 brick_coord = fract(vTexCoord * repeat + vec2(0.5, 0.0) * offset);
    //vec2 brick = smoothstep_impl(0.1, 0.2, brick_coord) - smoothstep(0.8, 0.9, brick_coord);
    //color = mix(grey, orange, min(brick.s, brick.t));

    // Task 8:  make a red filled circle in the center of the cube face (radius ≈ 0.3) with white background
    // Task 9:  repeat the circle in 5 times in both directions, shift even rows by 0.5 like in task 7
    //              try it on a teapot (see Task 9 in Cv5.java)
    vec2 repeat = vec2(5.0);
    float offset = 0.5 * floor(repeat.t * vTexCoord.t);
    vec2 granko_coord = fract(repeat * vTexCoord + vec2(offset, 0));
    float granko = smoothstep_impl(0.3, 0.32, length(granko_coord - 0.5));
    color = mix(red, white, granko);

    vec3 lighting = phong(color, color, materialSpecularColor, materialShininess);

    fragColor = vec4(lighting, 1.0);
}

/*
 * Computes lighting using Blinn-Phong model.
 */
vec3 phong(vec3 matAmbientColor, vec3 matDiffuseColor, vec3 matSpecularColor, float matShininess) {
    vec3 N = normalize(vNormal);

    vec3 lightDirection;
    if (lightPosition.w == 0.0) {
        lightDirection = normalize(lightPosition.xyz);
    } else {
        lightDirection = normalize(lightPosition.xyz - vPosition);
    }

    vec3 viewDirection = normalize(eyePosition - vPosition);
    vec3 halfVector = normalize(lightDirection + viewDirection);

    vec3 ambient = lightAmbientColor * matAmbientColor;

    float diffuseFactor = max(dot(N, lightDirection), 0.0);
    vec3 diffuse = lightDiffuseColor * matDiffuseColor * diffuseFactor;

    float specularFactor = pow(max(dot(N, halfVector), 0.0), matShininess) * diffuseFactor;
    vec3 specular = lightSpecularColor * matSpecularColor * specularFactor;

    return ambient + diffuse + specular;
}
